!--------1---------2---------3---------4---------5---------6---------7---------8
!
!  Module   output_mod
!> @brief   output data for analysis
!! @authors Norio Takase (NT)
!
!  (c) Copyright 2014 RIKEN. All rights reserved.
!
!--------1---------2---------3---------4---------5---------6---------7---------8

#ifdef HAVE_CONFIG_H
#include "../../config.h"
#endif

module output_mod

  use output_str_mod
  use fileio_control_mod
  use fileio_mod
  use string_mod
  use messages_mod

  implicit none
  private

  ! structures
  type, public :: s_out_info
    character(MaxFilename) :: angfile     = ''
    character(MaxFilename) :: cntfile     = ''
    character(MaxFilename) :: comangfile  = ''
    character(MaxFilename) :: comdisfile  = ''
    character(MaxFilename) :: comtorfile  = ''
    character(MaxFilename) :: coorfile    = ''
    character(MaxFilename) :: crsfile     = ''
    character(MaxFilename) :: disfile     = ''
    character(MaxFilename) :: enefile     = ''
    character(MaxFilename) :: fenefile    = ''
    character(MaxFilename) :: gprfile     = ''
    character(MaxFilename) :: grotopfile  = ''
    character(MaxFilename) :: grocrdfile  = ''
    character(MaxFilename) :: mapfile     = ''
    character(MaxFilename) :: pathcvfile  = ''
    character(MaxFilename) :: pcafile     = ''
    character(MaxFilename) :: pdbfile     = ''
    character(MaxFilename) :: pdb_avefile = ''
    character(MaxFilename) :: pdb_aftfile = ''
    character(MaxFilename) :: pmffile     = ''
    character(MaxFilename) :: prjfile     = ''
    character(MaxFilename) :: probfile    = ''
    character(MaxFilename) :: qntfile     = ''
    character(MaxFilename) :: rmsfile     = ''
    character(MaxFilename) :: rstfile     = ''
    character(MaxFilename) :: torfile     = ''
    character(MaxFilename) :: trjfile     = ''
    character(MaxFilename) :: trrfile     = ''
    character(MaxFilename) :: valfile     = ''
    character(MaxFilename) :: vcvfile     = ''
    character(MaxFilename) :: vecfile     = ''
    character(MaxFilename) :: velfile     = ''
    character(MaxFilename) :: weightfile  = ''
    character(MaxFilename) :: xscfile     = ''
  end type s_out_info

  ! subroutines
  public  :: show_ctrl_output
  public  :: read_ctrl_output
  public  :: setup_output

contains

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    show_ctrl_output
  !> @brief        show control parameters in OUTPUT section
  !! @authors      NT
  !! @param[in]    keys : comma separated keys for show
  !
  !======1=========2=========3=========4=========5=========6=========7=========8
  
  subroutine show_ctrl_output(keys)

    ! formal arguments
    character(*),            intent(in)    :: keys

    ! local variables
    integer                  :: i, ntok
    character(1000)          :: ckeys

    character(10), allocatable :: toks(:)


    if (len_trim(keys) == 0) &
      return

    ckeys = keys
    do i = 1, len(keys)
      if (ckeys(i:i) == ',') &
        ckeys(i:i) = ''
    end do

    ntok = split_num(ckeys)
    allocate(toks(ntok))
    call split(ntok, ntok, ckeys, toks)


    write(MsgOut,'(A)') '[OUTPUT]'

    do i = 1, ntok

      if (toks(i) == 'ang') &
        write(MsgOut,'(A)') 'angfile        = output.ang      # angle file'

      if (toks(i) == 'cnt') &
        write(MsgOut,'(A)') 'cntfile        = output.cnt      # CNT file'

      if (toks(i) == 'comang') &
        write(MsgOut,'(A)') 'comangfile     = output.comang   # COM angle file'

      if (toks(i) == 'comdis') &
        write(MsgOut,'(A)') 'comdisfile     = output.comdis   # COM distance file'

      if (toks(i) == 'comtor') &
        write(MsgOut,'(A)') 'comtorfile     = output.comtor   # COM torsion file'

      if (toks(i) == 'coor') &
        write(MsgOut,'(A)') 'coorfile       = output.coor     # coordinate restart file'

      if (toks(i) == 'crs') &
        write(MsgOut,'(A)') 'crsfile        = output.crs      # CRS file'

      if (toks(i) == 'dis') &
        write(MsgOut,'(A)') 'disfile        = output.dis      # distance file'

      if (toks(i) == 'ene') &
        write(MsgOut,'(A)') 'enefile        = output.ene      # REMD energy file'

      if (toks(i) == 'fene') &
        write(MsgOut,'(A)') 'fenefile       = output.fene     # free energy file'

!      if (toks(i) == 'gpr') &
!        write(MsgOut,'(A)') 'gprfile        = output.gpr      # go model parameter file'

      if (toks(i) == 'grotop') &
        write(MsgOut,'(A)') 'grotopfile     = output.grotop   # GROMACS topology file'

      if (toks(i) == 'grocrd') &
        write(MsgOut,'(A)') 'grocrdfile     = output.grocrd   # GROMACS coordinate file'

      if (toks(i) == 'map') &
        write(MsgOut,'(A)') 'mapfile        = output.map      # cryo-EM density map file'

      if (toks(i) == 'pathcv') &
        write(MsgOut,'(A)') 'pathcvfile     = output.pathcv   # PATH CV file'

      if (toks(i) == 'PATHCV') &
        write(MsgOut,'(A)') 'pathcvfile     = output{}.pathcv # PATH CV file'

      if (toks(i) == 'pca') &
        write(MsgOut,'(A)') 'pcafile        = output.pca      # PCA file'

      if (toks(i) == 'pdb') &
        write(MsgOut,'(A)') 'pdbfile        = output.pdb      # PDB file'

      if (toks(i) == 'pdb_ave') &
        write(MsgOut,'(A)') 'pdb_avefile    = output.pdb      # PDB file (Averaged coordinates of analysis atoms)'

      if (toks(i) == 'pdb_aft') &
        write(MsgOut,'(A)') 'pdb_aftfile    = output.pdb      # PDB file (Averaged coordinates of fitting atoms)'

      if (toks(i) == 'pmf') &
        write(MsgOut,'(A)') 'pmffile        = output.pmf      # potential of mean force file'

      if (toks(i) == 'prj') &
        write(MsgOut,'(A)') 'prjfile        = output.prj      # PRJ file'

      if (toks(i) == 'prob') &
        write(MsgOut,'(A)') 'probfile       = output.prob     # unbiased density of states file'

      if (toks(i) == 'qnt') &
        write(MsgOut,'(A)') 'qntfile        = output.qnt      # fraction of native contact file'

      if (toks(i) == 'rms') &
        write(MsgOut,'(A)') 'rmsfile        = output.rms      # RMSD file'

      if (toks(i) == 'rst') &
        write(MsgOut,'(A)') 'rstfile        = output.rst      # restart file'

      if (toks(i) == 'tor') &
        write(MsgOut,'(A)') 'torfile        = output.tor      # torsion file'

      if (toks(i) == 'TRJ') &
        write(MsgOut,'(A)') 'trjfile        = output{}.trj    # trajectory file'

      if (toks(i) == 'trj') &
        write(MsgOut,'(A)') 'trjfile        = output.trj      # trajectory file'

      if (toks(i) == 'trr') &
        write(MsgOut,'(A)') 'trrfile        = output.trr      # TRROT file'

      if (toks(i) == 'val') &
        write(MsgOut,'(A)') 'valfile        = output.val      # VAL file'

      if (toks(i) == 'vcv') &
        write(MsgOut,'(A)') 'vcvfile        = output.vcv      # Variance-Covarience Matrix file'

      if (toks(i) == 'vec') &
        write(MsgOut,'(A)') 'vecfile        = output.vec      # VEC file'

      if (toks(i) == 'vel') &
        write(MsgOut,'(A)') 'velfile        = output.vel      # velocity restart file'

      if (toks(i) == 'weight') &
        write(MsgOut,'(A)') 'weightfile     = output.weight   # weight file'

      if (toks(i) == 'WEIGHT') &
        write(MsgOut,'(A)') 'weightfile     = output{}.weight # weight file'

      if (toks(i) == 'xsc') &
        write(MsgOut,'(A)') 'xscfile        = output.xsc      # extended system file'

    end do

    write(MsgOut,'(A)') ' '

    deallocate(toks)

    return

  end subroutine show_ctrl_output

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    read_ctrl_output
  !> @brief        read control parameters in OUTPUT section
  !! @authors      NT
  !! @param[in]    handle   : unit number of control file
  !! @param[inout] out_info : OUTPUT section control parameters information
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine read_ctrl_output(handle, out_info)

    ! parameters
    character(*),            parameter :: Section = 'Output'

    ! formal argments
    integer,                 intent(in)    :: handle
    type(s_out_info),        intent(inout) :: out_info


    ! read parameters from control file
    !
    call begin_ctrlfile_section(handle, Section)

    call read_ctrlfile_string(handle, Section, 'Angfile',     &
         out_info%angfile)
    call read_ctrlfile_string(handle, Section, 'Cntfile',     &
         out_info%cntfile)
    call read_ctrlfile_string(handle, Section, 'Comangfile',  &
         out_info%comangfile)
    call read_ctrlfile_string(handle, Section, 'Comdisfile',  &
         out_info%comdisfile)
    call read_ctrlfile_string(handle, Section, 'Comtorfile',  &
         out_info%comtorfile)
    call read_ctrlfile_string(handle, Section, 'Coorfile',    &
         out_info%coorfile)
    call read_ctrlfile_string(handle, Section, 'Crsfile',     &
         out_info%crsfile)
    call read_ctrlfile_string(handle, Section, 'Disfile',     &
         out_info%disfile)
    call read_ctrlfile_string(handle, Section, 'Enefile',     &
         out_info%enefile)
    call read_ctrlfile_string(handle, Section, 'Fenefile',    &
         out_info%fenefile)
    call read_ctrlfile_string(handle, Section, 'Gprfile',     &
         out_info%gprfile)
    call read_ctrlfile_string(handle, Section, 'Grotopfile',  &
         out_info%grotopfile)
    call read_ctrlfile_string(handle, Section, 'Grocrdfile',  &
         out_info%grocrdfile)
    call read_ctrlfile_string(handle, Section, 'Mapfile',     &
         out_info%mapfile)
    call read_ctrlfile_string(handle, Section, 'Pathcvfile',  &
         out_info%pathcvfile)
    call read_ctrlfile_string(handle, Section, 'Pcafile',     &
         out_info%pcafile)
    call read_ctrlfile_string(handle, Section, 'Pdbfile',     &
         out_info%pdbfile)
    call read_ctrlfile_string(handle, Section, 'Pdb_avefile', &
         out_info%pdb_avefile)
    call read_ctrlfile_string(handle, Section, 'Pdb_aftfile', &
         out_info%pdb_aftfile)
    call read_ctrlfile_string(handle, Section, 'Pmffile',     &
         out_info%pmffile)
    call read_ctrlfile_string(handle, Section, 'Prjfile',     &
         out_info%prjfile)
    call read_ctrlfile_string(handle, Section, 'Probfile',    &
         out_info%probfile)
    call read_ctrlfile_string(handle, Section, 'Qntfile',     &
         out_info%qntfile)
    call read_ctrlfile_string(handle, Section, 'Rmsfile',     &
         out_info%rmsfile)
    call read_ctrlfile_string(handle, Section, 'Rstfile',     &
         out_info%rstfile)
    call read_ctrlfile_string(handle, Section, 'Torfile',     &
         out_info%torfile)
    call read_ctrlfile_string(handle, Section, 'Trjfile',     &
         out_info%trjfile)
    call read_ctrlfile_string(handle, Section, 'Trrfile',     &
         out_info%trrfile)
    call read_ctrlfile_string(handle, Section, 'Valfile',     &
         out_info%valfile)
    call read_ctrlfile_string(handle, Section, 'Vcvfile',     &
         out_info%vcvfile)
    call read_ctrlfile_string(handle, Section, 'Vecfile',     &
         out_info%vecfile)
    call read_ctrlfile_string(handle, Section, 'Velfile',     &
         out_info%velfile)
    call read_ctrlfile_string(handle, Section, 'weightfile',  &
         out_info%weightfile)
    call read_ctrlfile_string(handle, Section, 'Xscfile',     &
         out_info%xscfile)

    call end_ctrlfile_section(handle)


    ! write parameters to MsgOut
    !
    write(MsgOut,'(A)') 'Read_Ctrl_Output> Output Files'
    if (len_trim(out_info%angfile) > 0) &
      write(MsgOut,'(A20,A)') '  angfile         = ', trim(out_info%angfile)
    if (len_trim(out_info%cntfile) > 0) &
      write(MsgOut,'(A20,A)') '  cntfile         = ', trim(out_info%cntfile)
    if (len_trim(out_info%comangfile) > 0) &
      write(MsgOut,'(A20,A)') '  comangfile      = ', trim(out_info%comangfile)
    if (len_trim(out_info%comdisfile) > 0) &
      write(MsgOut,'(A20,A)') '  comdisfile      = ', trim(out_info%comdisfile)
    if (len_trim(out_info%comtorfile) > 0) &
      write(MsgOut,'(A20,A)') '  comtorfile      = ', trim(out_info%comtorfile)
    if (len_trim(out_info%coorfile) > 0) &
      write(MsgOut,'(A20,A)') '  coorfile        = ', trim(out_info%coorfile)
    if (len_trim(out_info%crsfile) > 0) &
      write(MsgOut,'(A20,A)') '  crsfile         = ', trim(out_info%crsfile)
    if (len_trim(out_info%disfile) > 0) &
      write(MsgOut,'(A20,A)') '  disfile         = ', trim(out_info%disfile)
    if (len_trim(out_info%enefile) > 0) &
      write(MsgOut,'(A20,A)') '  enefile         = ', trim(out_info%enefile)
    if (len_trim(out_info%fenefile) > 0) &
      write(MsgOut,'(A20,A)') '  fenefile        = ', trim(out_info%fenefile)
    if (len_trim(out_info%gprfile) > 0) &
      write(MsgOut,'(A20,A)') '  gprfile         = ', trim(out_info%gprfile)
    if (len_trim(out_info%grotopfile) > 0) &
      write(MsgOut,'(A20,A)') '  grotopfile      = ', trim(out_info%grotopfile)
    if (len_trim(out_info%grocrdfile) > 0) &
      write(MsgOut,'(A20,A)') '  grocrdfile      = ', trim(out_info%grocrdfile)
    if (len_trim(out_info%mapfile) > 0) &
      write(MsgOut,'(A20,A)') '  mapfile         = ', trim(out_info%mapfile)
    if (len_trim(out_info%pathcvfile) > 0) &
      write(MsgOut,'(A20,A)') '  pathcvfile      = ', trim(out_info%pathcvfile)
    if (len_trim(out_info%pcafile) > 0) &
      write(MsgOut,'(A20,A)') '  pcafile         = ', trim(out_info%pcafile)
    if (len_trim(out_info%pdbfile) > 0) &
      write(MsgOut,'(A20,A)') '  pdbfile         = ', trim(out_info%pdbfile)
    if (len_trim(out_info%pdb_avefile) > 0) &
      write(MsgOut,'(A20,A)') '  pdb_avefile     = ', trim(out_info%pdb_avefile)
    if (len_trim(out_info%pdb_aftfile) > 0) &
      write(MsgOut,'(A20,A)') '  pdb_aftfile     = ', trim(out_info%pdb_aftfile)
    if (len_trim(out_info%pmffile) > 0) &
      write(MsgOut,'(A20,A)') '  pmffile         = ', trim(out_info%pmffile)
    if (len_trim(out_info%prjfile) > 0) &
      write(MsgOut,'(A20,A)') '  prjfile         = ', trim(out_info%prjfile)
    if (len_trim(out_info%probfile) > 0) &
      write(MsgOut,'(A20,A)') '  probfile        = ', trim(out_info%probfile)
    if (len_trim(out_info%qntfile) > 0) &
      write(MsgOut,'(A20,A)') '  qntfile         = ', trim(out_info%qntfile)
    if (len_trim(out_info%rmsfile) > 0) &
      write(MsgOut,'(A20,A)') '  rmsfile         = ', trim(out_info%rmsfile)
    if (len_trim(out_info%rstfile) > 0) &
      write(MsgOut,'(A20,A)') '  rstfile         = ', trim(out_info%rstfile)
    if (len_trim(out_info%torfile) > 0) &
      write(MsgOut,'(A20,A)') '  torfile         = ', trim(out_info%torfile)
    if (len_trim(out_info%trjfile) > 0) &
      write(MsgOut,'(A20,A)') '  trjfile         = ', trim(out_info%trjfile)
    if (len_trim(out_info%trrfile) > 0) &
      write(MsgOut,'(A20,A)') '  trrfile         = ', trim(out_info%trrfile)
    if (len_trim(out_info%valfile) > 0) &
      write(MsgOut,'(A20,A)') '  valfile         = ', trim(out_info%valfile)
    if (len_trim(out_info%vcvfile) > 0) &
      write(MsgOut,'(A20,A)') '  vcvfile         = ', trim(out_info%vcvfile)
    if (len_trim(out_info%vecfile) > 0) &
      write(MsgOut,'(A20,A)') '  vecfile         = ', trim(out_info%vecfile)
    if (len_trim(out_info%velfile) > 0) &
      write(MsgOut,'(A20,A)') '  velfile         = ', trim(out_info%velfile)
    if (len_trim(out_info%weightfile) > 0) &
      write(MsgOut,'(A20,A)') '  weightfile      = ', trim(out_info%weightfile)
    if (len_trim(out_info%xscfile) > 0) &
      write(MsgOut,'(A20,A)') '  xscfile         = ', trim(out_info%xscfile)
    write(MsgOut,'(A)') ' '

    return

  end subroutine read_ctrl_output

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    setup_output
  !> @brief        setup output files
  !! @authors      NT
  !! @param[in]    out_info : OUTPUT section control parameters information
  !! @param[out]   output   : output information
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine setup_output(out_info, output)

    ! formal argments
    type(s_out_info),        intent(in)    :: out_info
    type(s_output),          intent(inout) :: output


    output%angfile     = out_info%angfile
    output%cntfile     = out_info%cntfile
    output%comangfile  = out_info%comangfile
    output%comdisfile  = out_info%comdisfile
    output%comtorfile  = out_info%comtorfile
    output%coorfile    = out_info%coorfile
    output%crsfile     = out_info%crsfile
    output%disfile     = out_info%disfile
    output%enefile     = out_info%enefile
    output%fenefile    = out_info%fenefile
    output%gprfile     = out_info%gprfile
    output%grotopfile  = out_info%grotopfile
    output%grocrdfile  = out_info%grocrdfile
    output%mapfile     = out_info%mapfile
    output%pathcvfile  = out_info%pathcvfile
    output%pcafile     = out_info%pcafile
    output%pdbfile     = out_info%pdbfile
    output%pdb_avefile = out_info%pdb_avefile
    output%pdb_aftfile = out_info%pdb_aftfile
    output%pmffile     = out_info%pmffile
    output%prjfile     = out_info%prjfile
    output%probfile    = out_info%probfile
    output%qntfile     = out_info%qntfile
    output%rmsfile     = out_info%rmsfile
    output%rstfile     = out_info%rstfile
    output%torfile     = out_info%torfile
    output%trjfile     = out_info%trjfile
    output%trrfile     = out_info%trrfile
    output%valfile     = out_info%valfile
    output%vcvfile     = out_info%vcvfile
    output%vecfile     = out_info%vecfile
    output%velfile     = out_info%velfile
    output%weightfile  = out_info%weightfile
    output%xscfile     = out_info%xscfile

    return

  end subroutine setup_output

end module output_mod
